import pandas as pd
import altair as alt

# make histograms for the numeric columns
def make_histogram(data, column: str, x_title: str, w=250, h=150):
    """ Using the data, column of interest, title, and proportion, returns a histogram.

    Parameters:
    ----------
    dataframe : pandas.DataFrame
                The dataframe with the data that will be used to make the histogram.
    column : str
             The column of the dataframe that will be on the x-axis of the resulting histogram.
    x_title : str
              The x-axis title of the resulting histogram.
    w : int, optional
        The width (in pixels) of the resulting histogram (the default is 250).
    h : int, optional
        The height (in pixels) of the resulting histogram (the default is 150).

    Returns:
    -------
    altair.vegalite.v5.api.Chart
        An altair histogram with the specified column on the x-axis and
        number of flights on the y-axis.

    Raises:
    ------
    TypeError
        If the input argument for:
        - data is not of type pandas.DataFrame
        - column is not of type str
        - w is not of type int
        - h is not of type int
    Exception
        If the input argument 
        - column is not in the inputted pandas Dataframe
    
    Examples:
    --------
    >>> import pandas as pd
    >>> import altair as alt
    >>> df = pd.read_csv('filename.csv') # Replace 'filename.csv' with your desired dataset filename
    >>> result = make_histogram(df, column_name, "x-axis title", w=200, h=200)
    >>> result

    """
    # check if the inputs are the correct type
    if type(data) is not pd.DataFrame:
        raise TypeError("You are not using a pandas DataFrame for the dataframe input")
    if type(column) is not str:
        raise TypeError("You are not using a string for the column input")
    if type(w) is not int:
        raise TypeError("You are not using an integer for the width (w) input")
    if type(h) is not int:
        raise TypeError("You are not using an integer for the height (h) input")
    
    # check if the specifed column is in the inputted pandas DataFrame
    if column not in data.columns:
        raise Exception("The specified column is not in the specified dataframe")
    
    numeric_plot = alt.Chart(data, width=w, height=h
              ).mark_bar(
              ).encode(
                  x=alt.X(column, title=x_title, bin=alt.Bin(maxbins=30)),
                  y=alt.Y("count()", title="Number of Flights")
                  )
    return numeric_plot


import pandas as pd
import pickle

def save_model(model, file_path):
    """Save a trained model to a file using pickle.

    Parameters:
    ----------
    - model: The trained model object to be saved.
    - file_path (str): The path to save the model file.

    Returns:
    -------
    None

    Examples:
    --------
    >>> import pandas as pd
    >>> import pickle
    >>> from sklearn.dummy import DummyClassifier
    >>> dummy_classifier = DummyClassifier(strategy = "stratified", random_state = 12)
    >>> save_model(dummy_classifier)
    """
    with open(file_path, 'wb') as f:
        pickle.dump(model, f)



import pandas as pd

# read CSV file and make a DataFrame
def read(file_name):
    """
    Using the relative path, reads the file and returns a data frame of said data.

    Parameters:
    ----------
    file_name: string containing the relative path to a CSV file that will be read.

    Returns:
    -------
    pd.DataFrame
        DataFrame containing the information read form the CSV file.

    Raises:
    ------
    FileNotFoundError:
        - If the file does not exist.
    
    pd.errors.EmptyDataError:
        - If the file is empty.
    
    Examples:
    --------
    >>> import pandas as pd
    >>> import altair as alt
    >>> df = read(filename) *replace filename with the name of the file and its relative path
    >>> df.print
    
    """
    return pd.read_csv(file_name)


import pandas as pd

def replace_value(dataframe, column, old_value, new_value):
    """
    Replaces a value in a column with a new value in a pandas DataFrame.
    Returns a pandas DataFrame.

    Parameters:
    ----------
    dataframe : pandas.DataFrame
                The dataframe with the data.
    column : str
             The column in which a value is to be changed.
    old_value : str, int, or float
                The value that will be replaced.
    new_value : str, int, or float
                The value that is the replacement.

    Returns:
    -------
    pandas.DataFrame
        A DataFrame with a new specified value, that replaced the old specified value,
        in the specified column

    Raises:
    ------
    TypeError
        If the input argument for:
        - dataframe is not of type pandas.DataFrame
        - column is not of type str
        - old_value is not of type str, int, or float
        - new_value is not of type str, int, or float  
    Exception
        If the input argument 
        - column is not in the inputted pandas Dataframe
        - old_value is not in the specified column
    
    Examples:
    --------
    >>> import pandas as pd
    >>> df = pd.read_csv('filename.csv') # Replace 'filename.csv' with your desired dataset filename
    >>> result = replace_value(df, column_name, "old value", "new value")
    >>> print(result)

    """
    # check if the inputs are the correct type
    if type(dataframe) is not pd.DataFrame:
        raise TypeError("You are not using a pandas DataFrame for the dataframe input")
    if type(column) is not str:
        raise TypeError("You are not using a string for the column input")
    if type(old_value) not in [str, int, float]:
        raise TypeError("You are not using a string, integer, or float for the old_value input")
    if type(new_value) not in [str, int, float]:
        raise TypeError("YYou are not using a string, integer, or float for the new_value input")
    
    # check if the specifed column is in the inputted pandas DataFrame
    if column not in dataframe.columns:
        raise Exception("The specified column is not in the specified dataframe")

    # check if old_value is in the specified column
    if old_value not in dataframe[column].values:
        raise Exception("The specified old_value is not in the specified column")

    dataframe.loc[dataframe[column] == old_value, column] = new_value
    return dataframe


import pandas as py

# filter the read CSV file and return columns of interest
def filter_columns(data, columns_of_interest):
    """
    Using an existing DataFrame and columns of interest, returns a DataFrame with only said columns of interest.

    Parameters:
    ----------
    data: pd.DataFrame 
        DataFrame containing the already read information that one wants to work with.

    columns_of_interest: [columns_of_interest]
        List containing strings of the names of the columns that you want to keep.

    Returns:
    -------
    pd.DataFrame
        DataFrame containing the information read form the CSV file.

    Raises:
    ------
    ValueError:
        - If the columns_of_interest list is empty.
        - If the data is empty
   
    KeyError:
        - If any of the specified columns do not exist in the DataFrame.
    
    TypeError:
        - If data is None.

    Examples:
    --------
    >>> df = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6], 'C': [7, 8, 9]})
    >>> filter_columns(df, ['A', 'B'])
       A  B
    0  1  4
    1  2  5
    2  3  6
    """
    # check if columns_of_interest is empty
    if len(columns_of_interest)==0:
        raise ValueError("the columns_of_interest list is empty")

    return data[columns_of_interest]


import pandas as pd
from sklearn.model_selection import train_test_split

def data_split(data, train_file_name, test_file_name):
    """
    Split the input DataFrame into train and test sets, and then save them into CSV files.

    Parameters:
    ----------
    data : pd.DataFrame
        Input DataFrame to be split.
        
    train_file_name : str
        Name of the CSV file to be saved to the train set.
        
    test_file_name : str
        Name of the CSV file to be saved to the test set.

    Returns:
    -------
    None

    Raises:
    ------
    TypeError:
        If the input data is not a DataFrame.
    ValueError:
        If either of the file names is empty.

    Examples:
    --------
    >>> data_split(input_data, 'train_data.csv', 'test_data.csv')
    """
    if not isinstance(data, pd.DataFrame):
        raise TypeError("Input data must be a pandas DataFrame.")
    if not train_file_name or not test_file_name:
        raise ValueError("File names cannot be empty.")

    # Split the data into train and test sets
    train, test = train_test_split(data, test_size=0.2, random_state=12)

    # Save the train and test sets into CSV files
    train.to_csv(train_file_name, index=False)
    test.to_csv(test_file_name, index=False)